import React, { PropTypes, Component } from 'react'
import { View } from 'react-native'
import lastElement from 'array-last'
import styled from 'styled-components/native'

import deepCopy from 'deep-copy'

import Toggle from './Toggle'

const GroupView = styled.View`
  flex: 1;
  flex-direction: ${props => props.mode === 'tag' ? 'row' : 'column'};
  ${props => props.mode === 'tag' ? 'flex-wrap: wrap; align-items: center;' : 'justify-content: center;'}
`

/**
 * Returns an array of ids which are selected
 * if arr = [{key: 1, value:true}, {key:2, value:false}, {key: 3, value: true}] and key = 'key'
 * it will return [1, 3]
 * @param arr
 * @param key
 * @param number whether return number or Array
 */
function getSelectedIds(arr, key, number = false) {
  const selected = []
  arr.forEach(val => {
    if (val.value) selected.push(val[key])
  })
  return number ? selected[0] : selected
}

function handleSingleSelect (arr, index) {
  return arr.map((val, i) => {
    const obj = val
    obj.value = (i === index) ? !val.value : false
    return obj
  })
}

function formatValue (props) {
  const { value, selectedIds, id } = props
  if (!selectedIds) return value
  const value$ = deepCopy(value)
  return value$.map(val => {
    if ((typeof selectedIds === 'object' && selectedIds.indexOf(val[id]) >= 0) ||
      (typeof selectedIds === 'number' && selectedIds === val[id])) val.value = true
    return val
  })
}

export default class Group extends Component {
  constructor (props) {
    super(props)

    this.formattedValue = formatValue(props)

    this.handleChange = this.handleChange.bind(this)
  }

  componentDidMount () {
    this.formattedValue = formatValue(this.props)
  }

  componentWillReceiveProps (newProps) {
    this.formattedValue = formatValue(newProps)
  }

  getElements () {
    const { type, mode, disabled } = this.props
    return this.formattedValue.map((val, i) => (
      <Toggle
        {...val}
        key={i}
        mode={mode}
        name={`${type}-${i}`}
        onChange={this.handleChange}
        type={type}
        value={val.value}
        disabled={disabled}
      />
    ))
  }

  handleChange (data) {
    const { name, type, id, value } = this.props
    let newValue = deepCopy(this.formattedValue)

    const index = parseInt(lastElement(data.name.split('-')), 10)

    if (type === 'checkbox' || type === 'switch') {
      newValue[index].value = data.value
    } else {
      newValue = handleSingleSelect(newValue, index)
    }

    this.props.onChange({
      index,
      name,
      oldValue: value,
      value: newValue,
      selectedIds: getSelectedIds(newValue, id, type === 'radio')
    })
  }

  render () {
    const { name, className, attributes, type, mode } = this.props
    return (
      <GroupView {...attributes} mode={mode}>
        {this.getElements()}
      </GroupView>
    )
  }
}

Group.propTypes = {
  attributes: PropTypes.object,
  className: PropTypes.string,
  mode: PropTypes.oneOf(['normal', 'tag']),
  name: PropTypes.string.isRequired,
  onChange: PropTypes.func.isRequired,
  type: PropTypes.oneOf([
    'radio', 'checkbox', 'switch'
  ]),
  value: PropTypes.arrayOf(
    PropTypes.shape({
      id: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
      count: PropTypes.number,
      label: PropTypes.string,
      value: PropTypes.bool
    })
  ),
  selectedIds: PropTypes.oneOfType([
    PropTypes.array,
    PropTypes.number
  ]),
  id: PropTypes.string,
  disabled: PropTypes.bool
}

Group.defaultProps = {
  id: 'id',
  disabled: false,
  selectedIds: []
}
